(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.react2vue = factory());
}(this, (function () { 'use strict';

function unwrapExports (x) {
	return x && x.__esModule ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var index$1 = createCommonjsModule(function (module, exports) {
"use strict";

exports.__esModule = true;

exports.default = function () {
  return {
    manipulateOptions: function manipulateOptions(opts, parserOpts) {
      parserOpts.plugins.push("jsx");
    }
  };
};

module.exports = exports["default"];
});

var jsx = unwrapExports(index$1);

var index$2 = createCommonjsModule(function (module, exports) {
"use strict";

exports.__esModule = true;

exports.default = function () {
  return {
    manipulateOptions: function manipulateOptions(opts, parserOpts) {
      parserOpts.plugins.push("classProperties");
    }
  };
};

module.exports = exports["default"];
});

var classProperties = unwrapExports(index$2);

var index$3 = createCommonjsModule(function (module, exports) {
"use strict";

exports.__esModule = true;

exports.default = function () {
  return {
    manipulateOptions: function manipulateOptions(opts, parserOpts) {
      parserOpts.plugins.push("objectRestSpread");
    }
  };
};

module.exports = exports["default"];
});

var restSpread = unwrapExports(index$3);

var isSpecialMethod = (function () {
  var specialMethods = new Set(['render', 'componentDidMount', 'componentWillMount', 'componentWillUnmount']);

  return function (methodName) { return specialMethods.has(methodName); }
})();

var mapMethodName = (function () {
  var map = null;
  return function (t, key) {
    if (!map) {
      map = {
        componentDidMount: t.identifier('mounted'),
        componentWillMount: t.identifier('beforeMount'),
        componentWillUnmount: t.identifier('beforeDestroy')
      };
    }
    return map[key.name] || key
  }
})();

var removeReactImport = function (t, path) {
  path.traverse({
    ImportDeclaration: function ImportDeclaration(path) {
      var specifiers = path.get('specifiers');
      var source = path.get('source');

      if (!t.isStringLiteral(source) || source.node.value !== 'react') {
        return
      }

      specifiers.forEach(function (specifier) {
        if (t.isImportDefaultSpecifier(specifier)) {
          specifier.remove();
          if (specifiers.length === 1) {
            path.remove();
          }
        }
      });
    }
  });
};

var getReactComponentIdentifier = function (t, path) {
  var result = null;

  path.traverse({
    ImportDeclaration: function ImportDeclaration(path) {
      var specifiers = path.get('specifiers');
      var source = path.get('source');

      if (
        specifiers.length === 1 &&
        t.isStringLiteral(source) &&
        source.node.value === 'react-dom' &&
        t.isImportDefaultSpecifier(specifiers[0]) &&
        specifiers[0].node.local.name === 'ReactDOM'
      ) {
        path.replaceWith(t.importDeclaration([t.importDefaultSpecifier(t.identifier('Vue'))], t.stringLiteral('vue')));
        return
      }

      if (!t.isStringLiteral(source) || source.node.value !== 'react') {
        return
      }

      specifiers.forEach(function (specifier) {
        if (!t.isImportSpecifier(specifier)) {
          return
        }

        var imported = specifier.get('imported');
        var local = specifier.get('local');

        if (t.isIdentifier(imported) && imported.node.name === 'Component' && t.isIdentifier(local)) {
          result = local.node.name;
          specifier.remove();
          if (specifiers.length === 1) {
            path.remove();
          }
        }
      });
    }
  });
  return result
};

var convertReactBody = function (t, path) {
  path.traverse({
    // this.state.* => this.$data.* and this.props.* => this.$attrs.*
    MemberExpression: function MemberExpression(path) {
      var object = path.get('object');
      var property = path.get('property');

      if (t.isThisExpression(object) && t.isIdentifier(property) && property.node.name === 'state') {
        property.replaceWith(t.identifier('$data'));
      } else if (t.isThisExpression(object) && t.isIdentifier(property) && property.node.name === 'props') {
        property.replaceWith(t.identifier('$attrs'));
      }
    },
    // className => class
    JSXAttribute: function JSXAttribute(path) {
      var name = path.get('name');
      if (t.isJSXIdentifier(name) && name.node.name === 'className') {
        path.replaceWith(t.jSXAttribute(t.jSXIdentifier('class'), path.get('value').node));
      }
    },
    // this.setState({...this.state, newProps: newVals})
    CallExpression: function CallExpression(path) {
      var callee = path.get('callee');
      var args = path.get('arguments');
      if (
        t.isMemberExpression(callee) &&
        t.isThisExpression(callee.get('object')) &&
        t.isIdentifier(callee.get('property')) &&
        callee.get('property').node.name === 'setState' &&
        args.length === 1 &&
        t.isObjectExpression(args[0])
      ) {
        var statePatch = args[0].get('properties');
        var toPatch = [];
        statePatch.forEach(function (property) {
          if (t.isSpreadElement(property)) {
            return
          }
          var key = property.get('key');

          toPatch.push({
            key: key,
            value: property.get('value')
          });
        });

        var assignments = toPatch.map(function (ref) {
            var key = ref.key;
            var value = ref.value;

            return t.expressionStatement(
            t.assignmentExpression(
              '=',
              t.memberExpression(t.thisExpression(), key.node, !t.isIdentifier(key.node)),
              value.node
            )
          );
        }
        );

        path.replaceWith(t.blockStatement(assignments));
      }
    }
  });
};

var convertReactComponent = function (t, path, isDefaultExport) {
  var id = path.get('id');
  var vueBody = [];

  var reactBody = path.get('body');
  var methods = [];

  reactBody.get('body').forEach(function (reactProperty) {
    var key = reactProperty.get('key');

    if (
      // normal methods
      t.isClassMethod(reactProperty) &&
      reactProperty.node.kind === 'method' &&
      t.isIdentifier(key)
    ) {
      var body = reactProperty.get('body');
      var params = reactProperty.node.params;
      convertReactBody(t, reactProperty.get('body'));
      var newMethod = t.objectMethod('method', mapMethodName(t, key.node), params, body.node);
      newMethod.async = reactProperty.node.async;
      newMethod.generator = reactProperty.node.generator;
      if (isSpecialMethod(key.node.name)) {
        vueBody.push(newMethod);
      } else {
        methods.push(newMethod);
      }
    } else if (
      // bound-to-class methods
      t.isClassProperty(reactProperty) &&
      !reactProperty.node.static &&
      !reactProperty.node.computed &&
      t.isArrowFunctionExpression(reactProperty.get('value'))
    ) {
      var arrowFn = reactProperty.get('value');
      var body$1 = arrowFn.get('body').node;
      if (!t.isBlockStatement(body$1)) {
        arrowFn.get('body').replaceWith(t.blockStatement([t.returnStatement(body$1)]));
        body$1 = arrowFn.get('body').node;
      }
      var params$1 = arrowFn.node.params;
      convertReactBody(t, arrowFn.get('body'));
      var newMethod$1 = t.objectMethod('method', mapMethodName(t, key.node), params$1, body$1);
      newMethod$1.async = arrowFn.node.async;
      newMethod$1.generator = arrowFn.node.generator;
      if (isSpecialMethod(key.node.name)) {
        vueBody.push(newMethod$1);
      } else {
        methods.push(newMethod$1);
      }
    } else if (
      // state
      t.isClassProperty(reactProperty) &&
      !reactProperty.node.static &&
      !reactProperty.node.computed &&
      t.isIdentifier(key) &&
      key.node.name === 'state'
    ) {
      vueBody.push(
        t.objectProperty(t.identifier('data'), t.arrowFunctionExpression([], reactProperty.get('value').node))
      );
    }
  });

  if (methods.length > 0) {
    vueBody.push(t.objectProperty(t.identifier('methods'), t.objectExpression(methods)));
  }

  if (isDefaultExport) {
    path.replaceWith(t.objectExpression(vueBody));
  } else {
    path.replaceWith(t.variableDeclaration('const', [t.variableDeclarator(id.node, t.objectExpression(vueBody))]));
  }
};

var index$4 = function (ref) {
  var t = ref.types;

  return {
    visitor: {
      Program: function Program(path) {
        removeReactImport(t, path);
        var componentIdentifier = getReactComponentIdentifier(t, path);
        var defaultExport = null;
        path.traverse({
          ExportDefaultDeclaration: function ExportDefaultDeclaration(path) {
            defaultExport = path.get('declaration');
          },
          ClassDeclaration: function ClassDeclaration(path) {
            var superClass = path.get('superClass');

            if (superClass && t.isIdentifier(superClass) && superClass.node.name === componentIdentifier) {
              convertReactComponent(t, path, path === defaultExport);
            }
          },
          CallExpression: function CallExpression(path) {
            var callee = path.get('callee');
            var object = callee.get('object');
            var property = callee.get('property');
            var computed = callee.node.computed;

            if (
              !computed &&
              t.isIdentifier(object) &&
              t.isIdentifier(property) &&
              object.node.name === 'ReactDOM' &&
              property.node.name === 'render'
            ) {
              var ref = path.get('arguments');
              var jsx = ref[0];
              var el = ref[1];
              path.replaceWith(
                t.newExpression(t.identifier('Vue'), [
                  t.objectExpression([
                    t.objectProperty(t.identifier('el'), el.node),
                    t.objectMethod(
                      'method',
                      t.identifier('render'),
                      [],
                      t.blockStatement([t.returnStatement(jsx.node)])
                    )
                  ])
                ])
              );
            }
          }
        });
      }
    }
  }
};

var index = {
  plugins: [
    jsx,
    classProperties,
    restSpread,
    index$4
  ]
};

return index;

})));
